message Biosample
 {
  uriorcurie id = 0
  string name = 0
  string description = 0
  fieldResearchSite collectedFrom = 0
 }
message CollectingBiosamplesFromSite
 {
  uriorcurie id = 0
  string name = 0
  string description = 0
  fieldResearchSite site = 0
 repeated  biosample biosamples = 0
 }
// A datastructure that can contain lists of instances from any selected classes in the schema
message DataListCollection
 {
 repeated  biosample biosampleList = 0
 repeated  fieldResearchSite frsList = 0
 repeated  collectingBiosamplesFromSite cbfsList = 0
 repeated  reifiedRelationship relationsList = 0
 }
// A site, outside of a laboratory, from which biosamples may be collected.
message FieldResearchSite
 {
  uriorcurie id = 0
  string name = 0
  string description = 0
 }
message ImmaterialEntity
 {
  uriorcurie id = 0
  string name = 0
  string description = 0
 }
message MaterialEntity
 {
  uriorcurie id = 0
  string name = 0
  string description = 0
 }
message MaterialSample
 {
  uriorcurie id = 0
  string name = 0
  string description = 0
 }
// A generic grouping for any identifiable entity
message NamedThing
 {
  uriorcurie id = 0
  string name = 0
  string description = 0
 }
message PlannedProcess
 {
  uriorcurie id = 0
  string name = 0
  string description = 0
 }
message ReifiedRelationship
 {
  namedThing subject = 0
  string predicate = 0
  namedThing object = 0
  string notes = 0
 }
message Site
 {
  uriorcurie id = 0
  string name = 0
  string description = 0
 }
message Specimen
 {
  uriorcurie id = 0
  string name = 0
  string description = 0
 }
